/* This file was generated by syncqt. */
#ifndef QT_QTMULTIMEDIA_VERSION_H
#define QT_QTMULTIMEDIA_VERSION_H

#define QTMULTIMEDIA_VERSION_STR "5.12.1"

#define QTMULTIMEDIA_VERSION 0x050C01

#endif // QT_QTMULTIMEDIA_VERSION_H
