/****************************************************************************
**
** Copyright (C) 2017 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the QtGui module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 3 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL3 included in the
** packaging of this file. Please review the following information to
** ensure the GNU Lesser General Public License version 3 requirements
** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 2.0 or (at your option) the GNU General
** Public license version 3 or any later version approved by the KDE Free
** Qt Foundation. The licenses are as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-2.0.html and
** https://www.gnu.org/licenses/gpl-3.0.html.
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef UIACONTROLTYPEIDS_H
#define UIACONTROLTYPEIDS_H

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API. It exists purely as an
// implementation detail. This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
//

#define UIA_ButtonControlTypeId         50000
#define UIA_CalendarControlTypeId       50001
#define UIA_CheckBoxControlTypeId       50002
#define UIA_ComboBoxControlTypeId       50003
#define UIA_EditControlTypeId           50004
#define UIA_HyperlinkControlTypeId      50005
#define UIA_ImageControlTypeId          50006
#define UIA_ListItemControlTypeId       50007
#define UIA_ListControlTypeId           50008
#define UIA_MenuControlTypeId           50009
#define UIA_MenuBarControlTypeId        50010
#define UIA_MenuItemControlTypeId       50011
#define UIA_ProgressBarControlTypeId    50012
#define UIA_RadioButtonControlTypeId    50013
#define UIA_ScrollBarControlTypeId      50014
#define UIA_SliderControlTypeId         50015
#define UIA_SpinnerControlTypeId        50016
#define UIA_StatusBarControlTypeId      50017
#define UIA_TabControlTypeId            50018
#define UIA_TabItemControlTypeId        50019
#define UIA_TextControlTypeId           50020
#define UIA_ToolBarControlTypeId        50021
#define UIA_ToolTipControlTypeId        50022
#define UIA_TreeControlTypeId           50023
#define UIA_TreeItemControlTypeId       50024
#define UIA_CustomControlTypeId         50025
#define UIA_GroupControlTypeId          50026
#define UIA_ThumbControlTypeId          50027
#define UIA_DataGridControlTypeId       50028
#define UIA_DataItemControlTypeId       50029
#define UIA_DocumentControlTypeId       50030
#define UIA_SplitButtonControlTypeId    50031
#define UIA_WindowControlTypeId         50032
#define UIA_PaneControlTypeId           50033
#define UIA_HeaderControlTypeId         50034
#define UIA_HeaderItemControlTypeId     50035
#define UIA_TableControlTypeId          50036
#define UIA_TitleBarControlTypeId       50037
#define UIA_SeparatorControlTypeId      50038
#define UIA_SemanticZoomControlTypeId   50039
#define UIA_AppBarControlTypeId         50040

#endif
