/****************************************************************************
**
** Copyright (C) 2017 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the QtGui module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 3 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL3 included in the
** packaging of this file. Please review the following information to
** ensure the GNU Lesser General Public License version 3 requirements
** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 2.0 or (at your option) the GNU General
** Public license version 3 or any later version approved by the KDE Free
** Qt Foundation. The licenses are as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-2.0.html and
** https://www.gnu.org/licenses/gpl-3.0.html.
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef UIAEVENTIDS_H
#define UIAEVENTIDS_H

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API. It exists purely as an
// implementation detail. This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
//

#define UIA_ToolTipOpenedEventId                                20000
#define UIA_ToolTipClosedEventId                                20001
#define UIA_StructureChangedEventId                             20002
#define UIA_MenuOpenedEventId                                   20003
#define UIA_AutomationPropertyChangedEventId                    20004
#define UIA_AutomationFocusChangedEventId                       20005
#define UIA_AsyncContentLoadedEventId                           20006
#define UIA_MenuClosedEventId                                   20007
#define UIA_LayoutInvalidatedEventId                            20008
#define UIA_Invoke_InvokedEventId                               20009
#define UIA_SelectionItem_ElementAddedToSelectionEventId        20010
#define UIA_SelectionItem_ElementRemovedFromSelectionEventId    20011
#define UIA_SelectionItem_ElementSelectedEventId                20012
#define UIA_Selection_InvalidatedEventId                        20013
#define UIA_Text_TextSelectionChangedEventId                    20014
#define UIA_Text_TextChangedEventId                             20015
#define UIA_Window_WindowOpenedEventId                          20016
#define UIA_Window_WindowClosedEventId                          20017
#define UIA_MenuModeStartEventId                                20018
#define UIA_MenuModeEndEventId                                  20019
#define UIA_InputReachedTargetEventId                           20020
#define UIA_InputReachedOtherElementEventId                     20021
#define UIA_InputDiscardedEventId                               20022
#define UIA_SystemAlertEventId                                  20023
#define UIA_LiveRegionChangedEventId                            20024
#define UIA_HostedFragmentRootsInvalidatedEventId               20025
#define UIA_Drag_DragStartEventId                               20026
#define UIA_Drag_DragCancelEventId                              20027
#define UIA_Drag_DragCompleteEventId                            20028
#define UIA_DropTarget_DragEnterEventId                         20029
#define UIA_DropTarget_DragLeaveEventId                         20030
#define UIA_DropTarget_DroppedEventId                           20031
#define UIA_TextEdit_TextChangedEventId                         20032
#define UIA_TextEdit_ConversionTargetChangedEventId             20033
#define UIA_ChangesEventId                                      20034

#endif
