/* This file was generated by syncqt. */
#ifndef QT_QT3DQUICKRENDER_VERSION_H
#define QT_QT3DQUICKRENDER_VERSION_H

#define QT3DQUICKRENDER_VERSION_STR "5.12.1"

#define QT3DQUICKRENDER_VERSION 0x050C01

#endif // QT_QT3DQUICKRENDER_VERSION_H
